# sb::Lock
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Lock;
use strict;
require 5.006;
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.07';
# 0.07 [2007/08/04] changed initialize
# 0.06 [2007/07/16] changed return to make them consistent
# 0.05 [2006/07/04] got rid of "return( undef )"
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2006/01/10] changed unlock and declare DESTROY
# 0.02 [2005/11/14] porting from Serene Bach
# 0.01 [2005/03/01] added locked_open, new as constructor / changed lock
# 0.00 [2005/02/04] porting from sblock.pl
# --------------------------------------
# // defines / static variables
sub LOCK_SH         (){ 1 }
sub LOCK_EX         (){ 2 }
sub LOCK_NB         (){ 4 }
sub LOCK_UN         (){ 8 }
sub DEFAULT_TIMEOUT (){ 20 }
sub DEFAULT_TRY_NUM (){ 10 }
my $aBasedir  = './lock/';
my $aBasename = 'lock';
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'dir'      => $self->basedir,
		'basename' => $self->basename,
		'timeout'  => 20,
		'trynum'   => 10,
		@_
	);
}
sub DESTROY
{
	my $self = shift;
	$self->unlock;
}
# --------------------------------------
# // public functions - class method
sub basedir
{
	my $class = shift;
	$aBasedir = shift if (@_);
	return $aBasedir;
}
sub basename
{
	my $class = shift;
	$aBasename = shift if (@_);
	return $aBasename;
}
# --------------------------------------
# // public functions - constructor
sub lock
{
	my $self = shift;
	$self = $self->new(@_) if (!ref($self));
	return $self->_global_lock;
}
# --------------------------------------
# // public functions - destructor
sub unlock
{ # from Perl memo / Copyright (C) 1999-2004 OHZAKI Hiroki
	my $self = shift;
	rename($self->{'current'}, $self->{'path'}) if ($self->{'current'} and $self->{'path'});
	$self->{'current'} = undef;
	$self->{'path'} = undef;
}
# --------------------------------------
# // public functions
sub locked_open
{
	my $self  = shift;
	my $file  = shift;
	my $clear = shift;
	my $handle; # my $handle = local *FILE; # for before Perl 5.005
	open($handle, "+<$file") or return;
	binmode($handle);
	eval { flock($handle, LOCK_EX); };
	if ($@)
	{
		# If your platform does not support flock, please implement another EXCLUSIVE LOCK here.
		# However, this module is already implemented another lock as _global_lock.
		# So probably there is nothing to do anymore.
	}
	truncate($handle, 0) if (!$clear);
	return $handle;
}
# --------------------------------------
# // private functions
sub _global_lock
{ # from Perl memo / Copyright (C) 1999-2004 OHZAKI Hiroki
	my $self = shift;
	my $name = $self->{'basename'};
	$self->{'path'} = $self->{'dir'} . $self->{'basename'};
	for (my $i = 0; $i < $self->{'trynum'}; $i++, sleep 1)
	{
		return $self if (rename($self->{'path'}, $self->{'current'} = $self->{'path'} . time));
	}
	opendir(LOCKDIR, $self->{'dir'});
	my @filelist = readdir(LOCKDIR);
	closedir(LOCKDIR);
	foreach (@filelist)
	{
		if (/^$name(\d+)/)
		{
			return $self if (
				time - $1 > $self->{'timeout'} 
				and rename($self->{'dir'} . $_, $self->{'current'} = $self->{'path'} . time)
			);
		}
	}
	return;
}
1;
=head1 NAME

sb::Lock - file lock module

=head1 SYNOPSIS

	use sb::Lock;
	my $lock = sb::Lock->lock('dir'=>'./lock/','basename'=>'lock');
	if ($lock)
	{
		# something to do....
		$lock->unlock;
		# actually it doesn't need to call "unlock",
		# "unlock" is called during destruction of a sb::Lock instance.
	}
	else
	{
		# error!
	}

=head1 AUTHOR

Takuya Otani <mailto:takuya.otani@gmail.com> http://serennz.sakura.ne.jp/
Original Script by OHZAKI Hiroki http://www.din.or.jp/~ohzaki/perl.htm

=head1 LICENSE

Copyright (C) 1999-2004 OHZAKI Hiroki
Copyright (C) 2004- T.Otani@SimpleBoxes and SerendipityNZ

=cut
__END__
