# sb::Plugin
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Plugin;
use strict;
use sb ();
use sb::Config ();
use sb::Language ();
use sb::Data ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.10';
# 0.10 [2007/11/30] modified register_admin_module to handle title_name method
# 0.09 [2007/11/30] modified load_admin_module to enable admin plugins
# 0.08 [2007/08/10] added read_plugin
# 0.07 [2007/08/08] modified for Serene Bach 3.*
# 0.06 [2006/11/07] changed _initialize to ignore debug code
# 0.05 [2006/10/11] changed register_plugin to set plugin param correctly
# 0.04 [2005/07/25] added 'receipt' to @pExtraTypes
# 0.03 [2005/07/24] chnaged register_content_module to allow multiple cms modules for a plugin
# 0.02 [2005/07/09] changed load_extra_module to handle extra plugins correctly
# 0.01 [2005/07/06] added register and loader for extra module to handle extra plugins.
# 0.00 [2005/02/01] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_PLUGIN_DIR (){ './plugin/' };
sub TEMPLATE_DIR       (){ 'resource/' };
my @ContentTypes = ('main','entry','comment','trackback','profile');
my @PluginTypes = ('counter','receipt'); # [memo] 'build' and 'update' will be added in the future.
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'dir' => DEFAULT_PLUGIN_DIR,
		'wid' => 0,
		@_
	);
	my $weblog = sb::Data->load('Weblog','id'=>$param{'wid'});
	$self->SUPER::initialize(
		'dir'    => (-d $param{'dir'} and -r $param{'dir'}) ? $param{'dir'} : undef,
		'set'    => ($weblog) ? $weblog->plugin : undef,
		'lists'  => [],
		'plugin' => {},
		'cms'    => {},
		'admin'  => {},
		'format' => {},
		'extra'  => {},
		'debug'  => undef,
		'error'  => undef,
	);
	$self->generate_accessor('dir','set','lists','plugin','debug','error');
	return $self;
}
# --------------------------------------
# // public functions - register
sub register_plugin
{
	my $class = shift;
	my %param = (
		'lang' => {},    # [optional][HASH] language settings
		'text' => {},    # [optional][HASH] plugin description
		'data' => undef, # [optional][SEL.] flag to use sb::Data::Plugin / 0:not use 1:use
		'file' => undef, # [optional][FILE] file name for plugin description
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	my ($name,$module) = _check_caller(caller);
	$self->{'plugin'}->{$name} = $self->_load_description(%param);
	$self->{'plugin'}->{$name}->{'lang'} = $param{'lang'};
	if ($param{'data'})
	{
		my $data = sb::Data->load('Plugin','cond'=>{'name'=>$name},'num'=>1);
		if (!$data)
		{
			$data = sb::Data->add('Plugin','name'=>$name);
			sb::Data->update($data) if ($data);
		}
		$self->{'plugin'}->{$name}->{'data'} = ($data) ? $data : undef;
	}
	return 1;
}
sub register_admin_module
{
	my $class = shift;
	my %param = (
		'mode'   => undef, # [required][CHAR] mode name
		'module' => undef, # [optional][CHAR] module name
		'level'  => 0,     # [optional][NUM.] authorization level / 0:everyone, 1:advanced user, 2:administrator
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	my ($name,$module) = _check_caller(caller);
	return if ($param{'mode'} eq '');
	return if ($self->ignore($name));
	$param{'module'} = $module if (!$param{'module'});
	$self->{'admin'}->{$name} = \%param;
	sb->get->lang->string('mode_' . $param{'mode'} => $self->{'plugin'}->{$name}->{'name'});
	if ( !defined(&{$param{'module'} . '::title_name'}) )
	{
		no strict 'refs';
		*{$param{'module'} . '::title_name'} = sub {
			return ' &gt; ' . sb->get->lang->string('mode_' . $param{'mode'});
		};
	}
	return 1;
}
sub register_content_module
{
	my $class = shift;
	my %param = (
		'type'     => undef, # [required][SEL.] indicated as @pContentType
		'callback' => undef, # [required][FUNC] reference of callback
		'field'    => undef, # [optional][CHAR] applied field if need be
		'name'     => undef, # [optional][CHAR] module name
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	my ($name,$module) = _check_caller(caller);
	return if (! $param{'type'} );
	return if (! grep($param{'type'},@ContentTypes) );
	return if (! $param{'callback'} or ref($param{'callback'}) ne 'CODE' );
	return if ( $self->ignore($name) );
	$name = $param{'name'}  if ($param{'name'} ne '');
	$param{'field'} = $name if ($param{'field'} eq '');
	$self->{'cms'}->{$name} = {
		'type'     => $param{'type'},
		'callback' => $param{'callback'},
		'field'    => $param{'field'},
	};
	return 1;
}
sub register_text_filter
{
	my $class = shift;
	my %param = (
		'name'     => undef, # [required][CHAR] name of text format
		'callback' => undef, # [required][FUNC] reference of callback
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	my ($name,$module) = _check_caller(caller);
	return if (! $param{'name'} or $param{'name'} =~ /\W/ );
	return if (! $param{'callback'} or ref($param{'callback'}) ne 'CODE' );
	return if ( $self->ignore($name) );
	$self->{'format'}->{$param{'name'}} = $param{'callback'};
	return 1;
}
sub register_extra_module
{
	my $class = shift;
	my %param = (
		'type'     => undef, # [required][SEL.] indicated as @PluginTypes
		'callback' => undef, # [required][FUNC] reference of callback
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	my ($name,$module) = _check_caller(caller);
	return if (! $param{'type'} );
	return if (! grep($param{'type'},@PluginTypes) );
	return if (! $param{'callback'} or ref($param{'callback'}) ne 'CODE' );
	return if ( $self->ignore($name) );
	if (! defined($self->{'extra'}->{$param{'type'}}) )
	{
		$self->{'extra'}->{$param{'type'}} = [ $param{'callback'} ];
	}
	else
	{
		push(@{$self->{'extra'}->{$param{'type'}}},$param{'callback'});
	}
	return 1;
}
# --------------------------------------
# // public functions - loader
sub load_admin_module
{
	my $class = shift;
	my $self = ref($class) ? $class : $class->get();
	my @list = ();
	my @names = sort { $a cmp $b } keys(%{$self->{'admin'}});
	foreach my $name ( @names )
	{
		my $util = $self->{'admin'}->{$name};
		push(@list,[$util->{'mode'},'util',$util->{'module'},$util->{'level'},'_']);
	}
	return @list;
}
sub load_content_module
{
	my $class = shift;
	my $parser = shift; # instance of sb::Content
	my $self = ref($class) ? $class : $class->get();
	my @names = sort { $a cmp $b } keys(%{$self->{'cms'}});
	foreach my $name ( @names )
	{
		my $module = $self->{'cms'}->{$name};
		$parser->{$module->{'type'}}{$module->{'field'}} = $module->{'callback'};
	}
	return;
}
sub load_text_filter
{
	my $class = shift;
	my $name = shift;
	my $self = ref($class) ? $class : $class->get();
	return if (! defined($self->{'format'}->{$name}) );
	return $self->{'format'}->{$name};
}
sub load_extra_module
{
	my $class = shift;
	my $type = shift;
	my $self = ref($class) ? $class : $class->get();
	return if (! grep($type,@PluginTypes) );
	return if (! defined($self->{'extra'}->{$type}) );
	return @{$self->{'extra'}->{$type}};
}
# --------------------------------------
# // public functions - utilities
sub load_template
{
	my $class = shift;
	my %param = (
		'file' => undef,
		'dir'  => '',
		@_
	);
	return if ($param{'file'} eq '');
	my $self = ref($class) ? $class : $class->get();
	$param{'dir'} = $self->template_dir() if ($param{'dir'} eq '');
	my $name = (_check_caller(caller))[0];
	my $lang = sb::Language->get;
	my $plugin_charcode = $self->{'plugin'}->{$name}->{'lang'}->{$lang->language};
	my $file = $param{'dir'} . $param{'file'};
	my $text = undef;
	my @template = ();
	if ( -r $file )
	{
		open(TEMPIN,"<$file") or die($lang->string('Failed to open the file') . $file);
		binmode(TEMPIN);
		@template = <TEMPIN>;
		close(TEMPIN);
	}
	$text = join('',@template);
	$text =~ s/\x0D\x0A/\n/g;
	$text =~ tr/\x0D\x0A/\n\n/;
	if ($plugin_charcode ne $lang->charcode)
	{
		$text = $lang->convert($text,$lang->charcode,$plugin_charcode);
	}
	return $text;
}
sub get_data
{
	my $class = shift;
	my $name = shift;
	my $self = ref($class) ? $class : $class->get();
	$name = (_check_caller(caller))[0] if ($name eq '');
	return $self->data($name);
}
sub set_data
{
	my $class = shift;
	my %param = (
		'name' => undef,
		'data' => undef,
		@_
	);
	my $self = ref($class) ? $class : $class->get();
	$param{'name'} = (_check_caller(caller))[0] if ($param{'name'} eq '');
	return if (! $self->{'plugin'}->{$param{'name'}} or ! defined($param{'data'}) );
	$self->{'plugin'}->{$param{'name'}}->{'data'} = $param{'data'};
	sb::Data->update($param{'data'});
	return 1;
}
# --------------------------------------
# // public functions - instance method
sub load_plugins
{
	my $self = shift;
	return if (!$self->dir());
	my $dir = $self->dir();
	# -- get files in plugin directory
	opendir(PLUGINDIR,$dir);
	my @files = readdir(PLUGINDIR);
	closedir(PLUGINDIR);
	# -- check through each files
	my @modules = ();
	@files = sort { $a cmp $b } @files;
	foreach my $file ( @files )
	{
		my $plugin = $dir . $file;
		next if (!-r $plugin or -d $plugin);
		next if ($file =~ /^\./ or $file =~ /^_/);
		next if ($file !~ /\.pl$/ and $file !~ /\.pm$/);
		push(@modules,$file);
		next if ($self->ignore($file));
		eval{ require $plugin; };
		$self->{'error'} .= $@ if ($self->debug() and $@)
	}
	$self->lists(\@modules);
	return;
}
sub read_plugin
{
	my $self = shift;
	my $name = shift;
	return if (!$self->dir() or !$name);
	my $file = $self->dir() . $name;
	my $lang = sb::Language->get;
	my $code = $lang->language;
	my $flag = undef;
	my %param = (
		'lang' => {},
		'text' => { # name and author will be read at least.
			'name'   => $name,
			'author' => '',
		},
		'file' => '',
	);
	return $param{'text'} if (!-r $file);
	open(PLUGIN,"<$file") or die($lang->string('Failed to open the file') . $file);
	binmode(PLUGIN);
	while (my $line = <PLUGIN>)
	{
		$flag = 1 if ($line =~ /register_plugin/);
		if ($flag)
		{
			$param{'lang'}->{$code} = $1    if ($line =~ /'\Q$code\E'.*?=>.*?'(.+?)'/);
			$param{'text'}->{'name'} = $1   if ($line =~ /'name'.*?=>.*?'(.+?)'/);
			$param{'text'}->{'author'} = $1 if ($line =~ /'author'.*?=>.*?'(.+?)'/);
			$param{'file'} = $1             if ($line =~ /'file'.*?=>.*?'(.+?)'/);
		}
		last if ($line =~ /\);/ and $flag);
	}
	close(PLUGIN);
	$param{'text'}->{'name'} = $name if (!$param{'text'}->{'name'});
	return ($param{'file'})
		? $self->_load_description(%param)
		: $param{'text'};
}
sub ignore
{
	my $self = shift;
	my $module = shift;
	return (index($self->set(),'[' . $module . ']') == -1);
}
sub data
{
	my $self = shift;
	my $name = shift;
	$name = (_check_caller(caller))[0] if ($name eq '');
	return if (! $self->{'plugin'}->{$name});
	return $self->{'plugin'}->{$name}->{'data'};
}
sub text_filters
{
	my $self = shift;
	return keys(%{$self->{'format'}});
}
sub resource_dir
{
	my $self = shift;
	return $self->dir() . TEMPLATE_DIR;
}
sub template_dir
{
	my $self = shift;
	return $self->resource_dir() . sb::Language->get->language . '/';
}
# --------------------------------------
# // public functions - deprecated
sub get_resource_dir
{ # the same as 'resource_dir', so we should use 'resource_dir' instead of this
	my $class = shift;
	my $self = ref($class) ? $class : $class->get();
	return $self->resource_dir();
}
sub get_list
{ # the same as 'plugin', so we should use 'plugin' instead of this
	my $class = shift;
	my $self = ref($class) ? $class : $class->get();
	return $self->plugin();
}
sub get_text_filter
{ # the same as 'text_filters', sow we should use 'text_filters' instead of this
	my $class = shift;
	my $self = ref($class) ? $class : $class->get();
	return $self->text_filters();
}
# --------------------------------------
# // private functions
sub _check_caller
{
	my @calls = @_; # $calls[0]:package, $calls[1]:filename, $calls[2]:line
	return ( _extract_filename($calls[1]), $calls[0] );
}
sub _extract_filename
{
	my $file = shift;
	$file =~ s/.*[:\/\\](.*)/$1/;
	$file =~ s/\[/&#91;/g;
	$file =~ s/\]/&#93;/g;
	return $file;
}
sub _decode
{
	my $data = shift;
	$data =~ s/\\(.)/$1 eq 't' and "\t" or $1 eq 'n' and "\n" or "$1"/eg;
	return $data;
}
sub _load_description
{
	my $self = shift;
	my %param = @_;
	my $lang = sb::Language->get;
	my $dir = $self->template_dir();
	if ($param{'file'} ne '' and $param{'lang'}->{$lang->language} ne '')
	{
		my $flag = ($param{'lang'}->{$lang->language} ne $lang->charcode);
		my $file = $dir . $param{'file'};
		return $param{'text'} if (!-r $file);
		open(TEXTIN,"<$file");
		binmode(TEXTIN);
		while (my $line = <TEXTIN>)
		{
			next if ($line =~ /^#/);
			$line =~ s/\x0D\x0A|\x0D|\x0A//g;
			$line = $lang->convert($line,$lang->charcode,$param{'lang'}->{$lang->language}) if ($flag);
			my ($field,$content) = split("\t",$line,2);
			if ($field eq 'resource')
			{
				my ($label,$string) = split("\t",$content,2);
				$lang->string(_decode($label)=>_decode($string));
			}
			else
			{
				$param{'text'}->{$field} = $content;
			}
		}
		close(TEXTIN);
	}
	return $param{'text'};
}
1;
__END__
